#!/usr/bin/perl

###########i#############################################
# Purpose: Rearrange the columns of the input pedfile
#          and the rows of input mapfile and datfile  
#          so that chromosomal order is perserved. Then, 
#          write the results to ord.ped, ord.dat, and 
#          ord.map.
#
#  syntax:               [pedfile] [datfile]  [mapfile]  
#  syntax:  ./reorder.pl  sub.ped   sub.dat    sub.map  
# 
#########################################################

use warnings;
$pedfile = $ARGV[0];
$datfile = $ARGV[1];
$mapfile = $ARGV[2];

open(IN,"$mapfile");
@MAP = <IN>;
close(IN);

# Make ord.map and compute the 1-indexed snp positions of the 
# input mapfile in chromosomal order
 
$header = shift(@MAP);
for( $i=0; $i < scalar(@MAP); $i++){ $MAP[$i] = join(" ",$i+1,$MAP[$i]); }

# (see below for an example)
@STUFF = map  { $_->[0] }
         sort { $a->[1] <=> $b->[1] }
         map  { [$_, (split)[-1]] } @MAP;

@ORDMAP = @INDXS = ();
for( $i=0; $i<scalar(@STUFF); $i++)
{
  @record = split(" ", $STUFF[$i]);
  push(@INDXS, $record[0]);
  push(@ORDMAP, $record[1]);
  push(@ORDMAP, $record[2]);
  push(@ORDMAP, $record[3]."\n");
}
open(OUT,">ord.map") || die(" There is not enough space to write to ord.map \n");
print OUT "$header";
print OUT " @ORDMAP";

# Use @INDXS to make ord.dat and ord.ped 

open(IN,"$datfile");
@DAT = <IN>;
@ORDDAT = @DAT;
close(IN);
open(IN,"$pedfile");
@PED = <IN>;
@ORDPED = @PED;
close(IN);

for($i=0; $i<scalar(@INDXS); $i++)   # for ea snp
{
  $w = $INDXS[$i]; 
  $ORDDAT[$i+1] = $DAT[$w];
}
for($i=0; $i<scalar(@PED); $i++)      # for ea person   
{
  @record = split(" ", $PED[$i]);
  @newrecord = @record[0..5];        # init newrec with relationship info 
  for($j=0; $j<scalar(@INDXS); $j++) # for ea snp within a person
  {
    $w = $INDXS[$j];
    $a1indx = 2*$w + 4;
    $a2indx = $a1indx + 1;
    push( @newrecord, $record[$a1indx] );
    push( @newrecord, $record[$a2indx] );
  }
  $ORDPED[$i] = join(" ", @newrecord)."\n";
}
open(OUT, ">ord.dat");
print OUT " @ORDDAT";
close(OUT);
open(OUT, ">ord.ped");
print OUT " @ORDPED";
close(OUT);

# Done
#
#
####################################################
# Miscellaneous  
#
# An understandable Swartzian Transform (ST) example 
# that sorts the array by its 6th column entry. 
# The most important thing about ST's is that 
# they are read from bottom to top.
#
# $str = "three 13  3  1  91 3
#         one   11  5  1  45 1
#         two   12  7  1  33 2
#         four  14  1  1  26 4";
#
# $str = join "",                      # take the refs returned by map and put them into a single string 
#        map { $_->[0]."\n" }          # for ea $_ in @_ (which is now sorted) assign the 0th element
#        sort { $a->[1] <=> $b->[1] }  # for ea address of #_ compare the 2nd element
#        map { [$_, (split)[-1]] }     # for ea $_ in @_ split on whtsp and return ($_, last element)
#        split /\n/, $str;             # make your string an array of strings
#
# print "$str\n";
#
# And the output:
# one   11  5  1  45 1
# two   12  7  1  33 2
# three 13  3  1  91 3
# four  14  1  1  26 4
