#!/usr/bin/perl

###########################################################################i####
#  Purpose: Extract genotypes and relationship information according to a set of 
#           selected markers (i.e. those that are labelled as 'M' in the 
#           datfile), and write the result to sub.ped, sub.dat, and sub.map 
#           respectively.
#  	    
#  syntax:              [pedfile] [datfile]  [mapfile]  
#  syntax:  ./xtract.pl  pedfile   datfile    mapfile  
# 
################################################################################

use warnings;
$pedfile = $ARGV[0];
$datfile = $ARGV[1];
$mapfile = $ARGV[2];
# Note: the order (above) is by convention. But, in extract.pl, the dat file 
#       must be dealt with first.


# Create sub.dat
open(IN, "$datfile") || die("Could not open file $datfile \n\n");
@DAT = <IN>;
close(IN);

@PINDXS = @DINDXS = @RSNAMES = @SUBDAT = ();
for ( $i=0; $i<scalar(@DAT); $i++)           # for each line in the datfile 
{
  if ($i == 0) { push (@SUBDAT, $DAT[$i]); } # keep the header

  @ISTHISM = split(" ", $DAT[$i]);
  if ($ISTHISM[0] eq 'M') 
  {
    $offset = 2*$i + 4;
    push (@PINDXS, $offset);                 # for ea selected snp, store the 0-indexed pedfile pos of the 1st allele
    push (@DINDXS, $i);                      # for ea selected snp, store the 0-indexed datfile pos 
    push (@RSNAMES, $ISTHISM[1]);            # for ea selected snp, store the snp name 
    push (@SUBDAT, $DAT[$i]);                # accumulate the selected snps
  }
  if ($i == (scalar(@DAT) - 1)) { push (@SUBDAT, $DAT[$i]);} # keep the footer
}

open(OUT, ">sub.dat") || die("There is not enough space to write to sub.dat \n\n");
foreach $line (@SUBDAT) { print OUT $line; }
close (OUT);


# Now make sub.ped
open(IN, "$pedfile") || die("Could not open file $pedfile \n\n");
@PED = <IN>;
close(IN);

open(OUT, ">sub.ped") || die("There is not enough space to write to sub.ped \n\n");
for ( $j=0; $j<scalar(@PED); $j++)           # for each person in the pedigree
{
  @subpedperson = ();
  @pedperson = split(" ", $PED[$j]);
                                             # retain relationship info
  for ( $k=0; $k<=5; $k++ ) { push (@subpedperson, $pedperson[$k]); }

  foreach $index (@PINDXS)                   # for ea selected snp, get this person's geno   
  { 
    push (@subpedperson, $pedperson[$index]);
    push (@subpedperson, $pedperson[$index+1]);
  } 
  print OUT "@subpedperson \n";              # build sub.ped one person at a time
}
close (OUT);


# Lastly, make sub.map
open(IN, "$mapfile") || die("Could not open file $mapfile \n\n");
@MAP = <IN>;
close(IN);

@SUBMAP = ($MAP[0]);                         # keep the map header 
$n = 0;
@LOOKFORMARKER = split(" ", $MAP[$n]);
chomp($LOOKFORMARKER[1]); 
for ($m=0; $m<scalar(@RSNAMES); $m++)        # for ea selected snp
{
  while ($RSNAMES[$m] ne $LOOKFORMARKER[1])  # find it in the mapfile
  {
    $n = $n + 1;
    @LOOKFORMARKER = split(" ", $MAP[$n]);
    chomp($LOOKFORMARKER[1]); 
  }
  push (@SUBMAP, $MAP[$n]);                  # and add it to sub.map
}

open(OUT, ">sub.map") || die("There is not enough space to write to sub.map. \n\n");
print OUT @SUBMAP;
close (OUT);

