/* LDinfo: Copyright 2009- : Columbia University. All rights       */
/* reserved. You may copy, use, modify, but NOT redistribute,      */
/* modified or otherwise.                                          */
/*                                                                 */
/* LDinfo takes a (standard 6-column) merlin pedigree file         */
/* containing densely spaced SNPs, and the corresponding merlin    */
/* dat and map files.  It returns the upper-triangular matrix of   */
/* pairwise LD estimates.  The estimates are obtained from         */
/* two-locus genotypes using the EM algorithm.                     */
/*                                                                 */
/* Note that the pedigree file cannot contain trailing blank lines */

#include "../Headers/globs.h"
#include "../Headers/ldinfo.h"
#include "../Headers/ldlib.h"

int main( int argc, char* argv[] )
{ 
  int dim[2] = {0,0}, i, j, ncol, newseed, nsnps, posld;
  float assoc, freqs[2] = {0,0};
  char *measure, pedfile[100] = "Peds/", ans[5]="";

  strcat( pedfile, argv[1]);
  measure = argv[2];
  posld = atoi(argv[3]);
  if( posld == 0 )
  {
    sprintf(ans,"No");
                            /* neg diseq will be estimated btwn  */
                            /* 1 and 1 alleles                   */
  }
  else sprintf(ans,"Yes");  /* estimate positive diseq only and  */
                            /* output the correct allele freqs   */
                            /* whenever a 2 allele is coupled    */
  if( !argv[4] ) 
  {            
    newseed = time(NULL);   /* if the seed was not set,          */
    srand( newseed );       /* choose a random seed now.         */
  }
  else 
  { 
    newseed = SEED;         /* set the seed to its default value */
    srand( newseed );
  }

  //pr_args( argv, newseed ); 
  //printf("\n reading pedigree...");
  //fflush(stdout);
  readped( pedfile, dim );                    /* read pedfile    */
  //printf("\n pedigree reading complete.");

  nsnps = (dim[1] - 6)/2;

  /* BEGIN PAIRWISE LD ESTIMATION */

  /*
  if( !strcmp(measure,"rsq") )
    printf("\n UPPER TRIANGULAR MATRIX OF PAIRWISE LD (%s; %d EM-Iterations) \n\n", measure, NEMit); 
  else
    printf("\n UPPER TRIANGULAR MATRIX OF PAIRWISE LD (|%s|; %d EM-Iterations) \n\n",measure, NEMit); 

  for( i=1; i <= nsnps; i++)    // (<=) --> (<) for upper tri
  {
    printf("row %d: \n",i);
    for( j=1; j <= nsnps; j++)  // 1 --> (i+1) for upper tri
    {
      assoc = ldm( measure, dim[0], posld, i, j, freqs );
      printf(" (%d, %d) : %6.4f \n", i, j, assoc);
    }
    printf("\n");
  }
  */

  if( !strcmp(measure,"rsq") )
    printf("\n ALL ADJACENT PAIRS (%s, %d EM-Iterations, coupled: %s) \n\n", measure,NEMit,ans);
  else
    printf("\n ALL ADJACENT PAIRS (|%s|, %d EM-Iterations, coupled: %s) \n\n",measure,NEMit,ans);

  for( i=1; i < nsnps; i++)
  {
    assoc = ldm( measure, dim[0], posld, i, (i+1), freqs );
    if( posld ) printf(" (%d, %d) : %6.4f  (%6.4f) \n", i, (i+1), assoc, freqs[0]);
    else printf(" (%d, %d) : %6.4f \n", i, (i+1), assoc);
  }
  if( posld ) printf("                      (%6.4f) \n\n", freqs[1]);
}
