/* LDsubsample: Copyright 2009- : Columbia University. All rights       */
/* reserved. You may copy, use, modify, but NOT redistribute, modified  */
/* or otherwise.                                                        */

/* LDsubsample takes a merlin pedigree containing densely spaced SNPs,  */
/* and the corresponding merlin dat and map files.  It returns a user   */
/* defined number of random subsamples (in the form of merlin dat files */ 
/* containing different patterns of M's and S2's).                      */

/*****************************************************************************************************************/
/* syntax : ./xldsubsamle  [pfile] [dfile] [mfile] [LD-measure] [LD-limit] [het] [Nsubs] [seed] [Nskip] [Uniqid] */
/* syntax : ./xldsubsample  x.ped   x.dat   x.map      rsq        .05       0.01   10    123     10      uniqid  */
/*****************************************************************************************************************/

#include "../Headers/globs.h"
#include "../Headers/ldsubsample.h"
#include "../Headers/ldlib.h"

int main( int argc, char* argv[] )
{
  errchk( argv );           /* check for command-line errors  */

  int nsubs  = atoi(argv[7]);
  int nskip  = atoi(argv[9]);
  int curr, i, k, ncol, nrow, newseed, nsnps, nobsnps;
  int snpspsubs, start, test, totsnps = 0;
  int dim[2] = {0,0};       /* number of rows and cols of (pedfile + genos) */
  float assoc, avghet, avgld, freqs[2] = {0,0}, hetdist[11], LB, ldmet=0, rfact, HET, UB, u, x, y;
  char pedfile[100] = "Peds/";
  char datfile[100] = "Dats/";
  char mapfile[100] = "Maps/";
  char dash[100], newdat[100], uniqid[100];
  char mode='r', *measure, index[20] = "";

  Nodeptr startptr;         /* startptr points to an end node, where */
                            /* node is a custom-built variable type  */
                            /* defined in ldsubsample.h              */
  
  if( argv[8] ) 
  {            
    newseed = atoi(argv[8]);
    srand( newseed );	    /* if the a random seed is given at the  */
                            /* given at the command-line, use it     */
  }
  else srand( SEED );       /* used the seed given in globs.h        */
 
  strcat(pedfile, argv[1]); 
  strcat(datfile, argv[2]); 
  strcat(mapfile, argv[3]); 
  measure = argv[4];
  avgld = atof(argv[5]);    /* this can be changed in globs.h, and 
                               relative to the population under 
                               study, it should represent the avg 
                               LD btwn adjacent pairs of SNPs in a 
                               sparse subsample. In terms of the 
                               program options in EAGLET it is the 
                               LD threshold parameter.              */
  HET = atof(argv[6]);
  rfact = RFACTOR;
  if( (1+rfact) * avgld > 1 ) rfact = (1-avgld)/avgld;
  LB = (1 - rfact) * avgld;
  UB = (1 + rfact) * avgld;

  readped( pedfile, dim );       /* read pedfile     */
  readdat( datfile, dim[1]);     /* read datfile     */ 
  nsnps  = (dim[1] - 6)/2;

  avghet = 0; for(k=0; k<11; k++) hetdist[k] = 0; nobsnps = 0;
  for(k=1; k<=nsnps; k++)
  {
    x = comphet( k, dim[0] );
    if( x != 999 )              /* this snp is obs in the sample   */
    {
      nobsnps++;
      if( x>= 0    && x < 0.05) hetdist[1]++;
      if( x>= 0.05 && x < 0.10) hetdist[2]++;
      if( x>= 0.10 && x < 0.15) hetdist[3]++;
      if( x>= 0.15 && x < 0.20) hetdist[4]++;
      if( x>= 0.20 && x < 0.25) hetdist[5]++;
      if( x>= 0.25 && x < 0.30) hetdist[6]++;
      if( x>= 0.30 && x < 0.35) hetdist[7]++;
      if( x>= 0.35 && x < 0.40) hetdist[8]++;
      if( x>= 0.40 && x < 0.45) hetdist[9]++;
      if( x>= 0.45 && x <= 0.5) hetdist[10]++;
      avghet += x;
    }
  }
  avghet /= (float) nobsnps;
  for(k=1; k<=10; k++) hetdist[k] /= (float) nsnps;

  for(k=1; k<=nsubs; k++)    /* for ea. subpanel */
  {
    
    /* Remark : Code to print without a line feed         */
    /*          printf("Generating subsample %6d \r", k); */ 
    /*          fflush(stdout); to empty the buffer       */

    startptr = NULL;
    snpspsubs = 0;

    start = getfirstsnp( dim, HET );         /* find 1st snp    */
    pushright( &startptr, start, ldmet );    /* push this snp   */
    snpspsubs++;                             /* onto the list   */

                   /* check proposed SNP for selection criteria */
    curr = start;
    test = curr + 1;
    while( test <= nsnps )
    {
      x = comphet(test, dim[0]);                                                   // h_i
      if( (test - curr) >= nskip && x != 999) /* skip close or unobs'd snps */
      {
        if( x > avghet) y = avgld + ( (x-avghet) * (UB-avgld) / (.5-avghet) );     // y is true worth: higher here than avg worth
        if( x <= avghet) y = avgld + ( (x-avghet) * (avgld-LB) / avghet );         //                  lower here than avg worth 
        assoc = ldm( measure, dim[0], 0, curr, test, freqs );
        u = (float) rand() / (float) RAND_MAX;
        if( assoc <= y )
        {
          y = (y - assoc)/y;                 /* the percent savings in LD   */
          i = 0; hetdist[0] = 0;
          while( x >= hetdist[0] )           /* find the het class          */
          {
            i++;
            hetdist[0] += 0.05;
          }
          if( hetdist[i] >= 0.1 ) x = 0.4 * sqrt(y) + 0.5;
          else x = 0.1 * sqrt(y) + 0.9;
          if( u < x )
          {
            pushright( &startptr, test, assoc );
            snpspsubs++;
            curr = test;
          }
        }
      }
      test++;
    }
    test = start - 1;
    curr = start;
    while( test >= 1 )
    {
      x = comphet(test, dim[0]);
      if( (curr - test) >= nskip && x != 999) /* skip close or unobs'd snps  */
      {
        if( x > avghet) y = avgld + ( (x-avghet) * (UB-avgld) / (.5-avghet) );
        if( x <= avghet) y = avgld + ( (x-avghet) * (avgld-LB) / avghet );
        assoc = ldm( measure, dim[0], 0, curr, test, freqs );
        u = (float) rand() / (float) RAND_MAX;
        if( assoc <= y )
        {
          y = (y - assoc)/y;           /* the percent savings in LD     */
          i = 0; hetdist[0] = 0;
          while( x >= hetdist[0] )     /* find the heterozygosity class */
          {
            i++;
            hetdist[0] += 0.05;
          }
          if( hetdist[i] >= 0.1 ) x = 0.4 * sqrt(y) + 0.5;
          else x = 0.1 * sqrt(y) + 0.9;
          if( u < x )
          {
            pushleft( &startptr, test, assoc );
            snpspsubs++;
            curr = test;
          }
        }
      }
      test--;
    }

    fflush( stdout );
    totsnps += snpspsubs;
  
    sprintf(index, "%d", k);
    sprintf(newdat, "%s", "Results/Subsample/Subs/sub-");
    strcat(newdat, argv[10]);
    sprintf(dash, "%s", "-");
    strcat(newdat, dash);
    strcat(newdat, index);

    /* write new merlin datfile    */ 
    if( snpspsubs < 2 ) writedat( newdat, dim[1], startptr, 0 ); 
    else writedat( newdat, dim[1], startptr, 1 );    
    delete( &startptr );

    /* Remark : fork() interacts with the buffer and with open i/o streams. */
    /*        therefore, the buffer should be emptied via fflush(), and if  */
    /*        possible, all open custom-designed streams should be closed.  */
    /*        moreover, fork() and execl() are often used in tandem,        */
    /*        because a call to execl() without calling fork() first        */
    /*        results in the creation of a 'ungrateful' child process that  */
    /*        completely destroys any record of the parent process.         */
    /*                                                                      */
    /*        Furthermore, in order to use functions like fork(), pipe(),   */
    /*        write(), open(), close() etc., you must include unistd.h, and */
    /*        while this header file works for Unix, Linux, and Mac OS X,   */
    /*        it is operating system specific.                              */
    /*                                                                      */ 
    /*        To communicate using a pipe, one must employ random file      */
    /*        access functions explicitly in the parent process, and should */
    /*        recall that the child process uses sequential access          */
    /*        functions by default.                                         */

  }
  
  pr_sumstats( dim, argv, assoc, totsnps, nsubs );  

  freemem( dim ); 
}
