/* LDsubsample: Copyright 2009- : Nationwide Children's Hospital and    */ 
/* The Ohio State University. All rights reserved. You may copy, use,   */
/* modify, but NOT redistribute, modified or otherwise.                 */

/* This version of LDsubsample, 'ldsubsample.rand.c' takes a merlin     */
/* pedigree containing densely spaced SNPs, and the corresponding       */
/* merlin dat and map files.  It returns a user defined number of       */
/* random subsamples in the form of merlin dat files containing         */ 
/* different patterns of M's and S2's. Here, the subsampling does not   */
/* account (explicitly) for LD. As such, the linked list is no longer   */ 
/* needed, and memory management is greatly simplified for use with     */ 
/* Bash Reduce.                                                         */

/*****************************************************************************************************************/
/* syntax : ./xldsubsamle  [pfile] [dfile] [mfile] [LD-measure] [LD-limit] [het] [Nsubs] [seed] [Nskip] [Uniqid] */
/* syntax : ./xldsubsample  x.ped   x.dat   x.map      rsq        .05       0.01   10    123     10      uniqid  */
/*****************************************************************************************************************/

#include "../Headers/globs.h"
#include "../Headers/ldsubsample.h"
#include "../Headers/ldlib.h"

int main( int argc, char* argv[] )
{
  errchk( argv );           /* check for command-line errors                */

  int cols, i, k, newseed, nselsnps, nsnps, rows, total; 
  int nsubs  = atoi(argv[7]);
  int nskip  = atoi(argv[9]);
  int dim[2] = {0,0};       /* number of rows and cols of (pedfile + genos) */
  char pedfile[100] = "Peds/";
  char datfile[100] = "Dats/";
  char mapfile[100] = "Maps/";
  char dash[100], index[20], newdat[100], uniqid[100];
  float assoc = 1, p, u;

  if( argv[8] ) 
  {            
    newseed = atoi(argv[8]);
    srand( newseed );	    /* if the a random seed is given at the  */
                            /* given at the command-line, use it     */
  }
  else srand( SEED );       /* used the seed given in globs.h        */
 
  strcat(pedfile, argv[1]); 
  strcat(datfile, argv[2]); 
  strcat(mapfile, argv[3]); 

  readped( pedfile, dim );   /* read pedfile                               */
  readdat( datfile, dim[1]); /* read datfile                               */ 
  cols     = dim[1];
  nsnps    = (cols - 6)/2;   /* number of columns minus six divided by two */
  nselsnps = nsnps/nskip;    /* number of selected snps per subsample      */
  p        = (double)nselsnps/(double)nsnps; 

  for(k=1; k<=nsubs; k++)    /* for ea. subsample */
  {
    
    /* Remark : Code to print without a line feed         */
    /*          printf("Generating subsample %6d \r", k); */ 
    /*          fflush(stdout); to empty the buffer       */

    sprintf(index, "%d", k);
    sprintf(newdat, "%s", "Results/Subsample/Subs/sub-");
    strcat(newdat, argv[10]);
    sprintf(dash, "%s", "-");
    strcat(newdat, dash);
    strcat(newdat, index);

    /*********************************************************/
    /* Write the new (i.e. sparse) merlin datfile to a file  */
    /* This code assumes that the old (i.e. dense) merlin    */
    /* datfile contains only M's                             */
    /*********************************************************/

    FILE* fptr;

    fptr = fopen(newdat, "w");
    rows = (cols - 6)/2 + 2;         /* number of rows in the (dense) datfile */

    for( i=0; i<rows; i++)           /* for each row of the (dense) datfile   */
    {
      u = (float) rand() / (float) RAND_MAX;      
      if( i < 2 || i > (rows-3) ) fprintf(fptr, "%-5s %-10s \n", DAT[i][0], DAT[i][1]);
       
      if( i >= 2 && i <= (rows-3) )  /* this is an interior snp               */
      {
        if( u > p ) fprintf(fptr, "%-5s %-10s \n", "S2 ", DAT[i][1]);
        else fprintf(fptr, "%-5s %-10s \n", DAT[i][0], DAT[i][1]);
      }
    }
    fclose(fptr);

    /* Remark : fork() interacts with the buffer and with open i/o streams. */
    /*        therefore, the buffer should be emptied via fflush(), and if  */
    /*        possible, all open custom-designed streams should be closed.  */
    /*        moreover, fork() and execl() are often used in tandem,        */
    /*        because a call to execl() without calling fork() first        */
    /*        results in the creation of a 'ungrateful' child process that  */
    /*        completely destroys any record of the parent process.         */
    /*                                                                      */
    /*        Furthermore, in order to use functions like fork(), pipe(),   */
    /*        write(), open(), close() etc., you must include unistd.h, and */
    /*        while this header file works for Unix, Linux, and Mac OS X,   */
    /*        it is operating system specific.                              */
    /*                                                                      */ 
    /*        To communicate using a pipe, one must employ random file      */
    /*        access functions explicitly in the parent process, and should */
    /*        recall that the child process uses sequential access          */
    /*        functions by default.                                         */

  }
  
  total = nsubs * nselsnps;
  pr_sumstats( dim, argv, assoc, total, nsubs );  

  freemem( dim );
}
