/* Create a bi-directional linked list of integers,  */
/* where each integer represents a SNP chosen at     */
/* random according to P*(S) along the chromosome.   */
/*                                                   */
/*                  1. pushright(a,b)                */
/*                  2. pushleft(a,b)                 */
/*                  3. delete(a)                     */
/*                  4. pr_list(a)                    */

#include <stdlib.h>
#include <stdio.h>
#include "../Headers/ldsubsample.h"


/******************************************/
/* Push an integer onto the rightmost end */
/* of the linked list. Since sptr is      */
/* passed by reference, its value can be  */
/* changed                                */
/******************************************/

void pushright( Nodeptr *sptr, int indx, float ldmet )
{
  Nodeptr newptr, prevptr, currptr;
  newptr = malloc( sizeof(Node) );

  if( newptr != NULL )  /* memory is available */
  {
    newptr->snpindx = indx;
    newptr->nxtrightptr = NULL;
    newptr->nxtleftptr  = NULL;
    newptr->ldmetricl   = ldmet;

    prevptr = NULL;
    currptr = *sptr;  /* at the first (and only the first)   */
                      /* call to pushright, *sptr, prevptr,  */
                      /* and currptr are all NULL            */

    while( currptr != NULL ) /* walk right to the next Node  */
    {
      prevptr = currptr;
      currptr = currptr->nxtrightptr;
    }

    if( prevptr == NULL ){ newptr->nxtrightptr = *sptr; }
    else
    {
      prevptr->nxtrightptr = newptr;
      prevptr->ldmetricr = ldmet;
      newptr->nxtrightptr = currptr;
    }
    
    *sptr = newptr;     /* Change the value of a pointer by */
                        /* reference                        */
    newptr->nxtleftptr = prevptr;
  }
  else
  {
    fprintf(stderr, "Error : Out of memory. \n");
    exit(8);
  }
}


/******************************************/
/* Push an integer onto the leftmost end  */
/* of the linked list.                    */
/******************************************/

void pushleft( Nodeptr *sptr, int indx, float ldmet )
{
  Nodeptr newptr, prevptr, currptr;

  newptr = malloc( sizeof(Node) );

  if( newptr != NULL )  /* memory is available */
  {
    newptr->snpindx = indx;
    newptr->nxtrightptr = NULL;
    newptr->ldmetricr   = ldmet;
    newptr->nxtleftptr  = NULL;

    prevptr = NULL;
    currptr = *sptr;  
    
    while( currptr != NULL ) /* walk left to the next Node */
    {
      prevptr = currptr;
      currptr = currptr->nxtleftptr;
    }

    if( prevptr == NULL ){ newptr->nxtleftptr = *sptr; }
    else
    {
      prevptr->nxtleftptr = newptr;
      prevptr->ldmetricl  = ldmet;
      newptr->nxtleftptr  = currptr;
    }

    *sptr = newptr;
    newptr->nxtrightptr = prevptr;
  }
  else
  {
    fprintf(stderr, "Error : Out of memory. \n");
    exit(8);
  }
}


/*****************************************/
/* Free the memory for each node of the  */
/* linked list.                          */
/*****************************************/

void delete( Nodeptr *currptr )
{
  Nodeptr tmpptr;

  if( (*currptr) == NULL ) printf(" Linked list is empty.\n");
  else
  {
    while( (*currptr)->nxtrightptr != NULL) (*currptr) = (*currptr)->nxtrightptr;

    while( (*currptr) != NULL )
    {
      tmpptr = (*currptr);
      (*currptr) = (*currptr)->nxtleftptr;
      free(tmpptr);
    }
  }
}


/**************************************/
/* Print out the complete linked list */
/**************************************/

void pr_list( Nodeptr currptr )
{
  if( currptr == NULL ) printf(" Linked list is empty.\n");
  else
  {
    while( currptr->nxtleftptr != NULL) currptr = currptr->nxtleftptr; 

    printf("\n\n right-SNP  LD-measure  left-SNP \n");
    while( currptr != NULL )
    {
      if( currptr->nxtrightptr ) printf("%5d", currptr->snpindx);
      currptr = currptr->nxtrightptr;
      if( currptr )
        printf("      %9.4f    %5d", currptr->ldmetricl, currptr->snpindx );
      printf("\n");
    }
  }
}

