#include <stdio.h>
#include <string.h>
#include <math.h>
#include "globs.h"

/*           The functions in this file are:                    */
/*                1. ldm()                                      */
/*                2. comphet()                                  */

/****************************************************************/
/* Use NEMit iterations of the EM algorithm to compute rsquared */
/* (if flag equal 'rsq') or dprime otherwise, between two       */
/* SNPs.  The two-locus genotypic counts are also computed at   */
/* this time.                                                   */
/****************************************************************/

float ldm( char* flag, int row, int posld, int snp1, int snp2, float* frqs )
{
  
  int a1f, a2f, a1s, a2s, i, j, nsnps, tcstat;
  static int cstat = 1;      /* keeps track of the allele at */
                             /* the 2nd locus that is        */
                             /* coupled with the '1' allele  */ 
                             /* at the 1st locus. cstat is   */ 
                             /* initialized once, no matter  */
                             /* how often ldm() is called.   */

  float D, Dmax, Dmaxpos, Dmaxneg, mgcounts[10];
  float p11, p12, p21, p22;  /* initial freq estimates       */
  float ss;                  /* sample size (in individuals) */
  float tmp, p1w, pw1, ld;

  for( i=0; i<=9; i++)
    mgcounts[i] = 1.0/9.0;  /* initialize two-locus          */
                            /* genotypic class counts using  */
                            /* pseudo-counts to avoid issues */
                            /* on the boundary.              */

  a1f = 2 * snp1 + 5;
  a2f = a1f + 1;
  a1s = 2 * snp2 + 5;
  a2s = a1s + 1;

  for( i=1; i<=row; i++ )
  {
    if( PED[i][a1f]==1 && PED[i][a2f]==1 && PED[i][a1s]==1 && PED[i][a2s]==1 ) mgcounts[1]++;
    if( PED[i][a1f]==1 && PED[i][a2f]==1 && PED[i][a1s]==1 && PED[i][a2s]==2 ) mgcounts[2]++;
    if( PED[i][a1f]==1 && PED[i][a2f]==1 && PED[i][a1s]==2 && PED[i][a2s]==1 ) mgcounts[2]++;
    if( PED[i][a1f]==1 && PED[i][a2f]==1 && PED[i][a1s]==2 && PED[i][a2s]==2 ) mgcounts[3]++;

    if( PED[i][a1f]==1 && PED[i][a2f]==2 && PED[i][a1s]==1 && PED[i][a2s]==1 ) mgcounts[4]++;
    if( PED[i][a1f]==2 && PED[i][a2f]==1 && PED[i][a1s]==1 && PED[i][a2s]==1 ) mgcounts[4]++;
    if( PED[i][a1f]==1 && PED[i][a2f]==2 && PED[i][a1s]==1 && PED[i][a2s]==2 ) mgcounts[5]++;
    if( PED[i][a1f]==1 && PED[i][a2f]==2 && PED[i][a1s]==2 && PED[i][a2s]==1 ) mgcounts[5]++;
    if( PED[i][a1f]==2 && PED[i][a2f]==1 && PED[i][a1s]==1 && PED[i][a2s]==2 ) mgcounts[5]++;
    if( PED[i][a1f]==2 && PED[i][a2f]==1 && PED[i][a1s]==2 && PED[i][a2s]==1 ) mgcounts[5]++;
    if( PED[i][a1f]==1 && PED[i][a2f]==2 && PED[i][a1s]==2 && PED[i][a2s]==2 ) mgcounts[6]++;
    if( PED[i][a1f]==2 && PED[i][a2f]==1 && PED[i][a1s]==2 && PED[i][a2s]==2 ) mgcounts[6]++;

    if( PED[i][a1f]==2 && PED[i][a2f]==2 && PED[i][a1s]==1 && PED[i][a2s]==1 ) mgcounts[7]++;
    if( PED[i][a1f]==2 && PED[i][a2f]==2 && PED[i][a1s]==1 && PED[i][a2s]==2 ) mgcounts[8]++;
    if( PED[i][a1f]==2 && PED[i][a2f]==2 && PED[i][a1s]==2 && PED[i][a2s]==1 ) mgcounts[8]++;
    if( PED[i][a1f]==2 && PED[i][a2f]==2 && PED[i][a1s]==2 && PED[i][a2s]==2 ) mgcounts[9]++;
  }

  /* a convenient type of mg notation */
  float n1111=mgcounts[1], n1112=mgcounts[2],  n1122=mgcounts[3];
  float n1211=mgcounts[4], n1212=mgcounts[5],  n1222=mgcounts[6];
  float n2211=mgcounts[7], n2212=mgcounts[8],  n2222=mgcounts[9];

  /* print out the counts */
  #if 0
  printf("%3.0f, %3.0f, %3.0f \n", n1111, n1112, n1122);
  printf("%3.0f, %3.0f, %3.0f \n", n1211, n1212, n1222);
  printf("%3.0f, %3.0f, %3.0f \n", n2211, n2212, n2222);
  #endif

  ss = n1111 + n1112 + n1122 + n1211 + n1212 + n1222 + n2211 + n2212 + n2222;

  /* marginal mles assumed fixed */
  p1w = (n1111 + n1112 + n1122 + .5*(n1211 + n1212 + n1222))/ss;
  pw1 = (n1111 + n1211 + n2211 + .5*(n1112 + n1212 + n2212))/ss;

  /* initialize the pis and the pijs */
  p11 = (n1111 + .5*n1112 + .25*n1212 + .5*n1211)/ss;
  p12 = p1w - p11;
  p21 = pw1 - p11;
  p22 = 1 - p11 - p12 - p21;

  Dmaxpos = p1w * (1-pw1);
  if( ((1-p1w)*pw1) < Dmaxpos ) Dmaxpos = ((1-p1w)*pw1);
  Dmaxneg = p1w * pw1;
  if( ((1-pw1)*(1-p1w)) < Dmaxneg ) Dmaxneg = (1-pw1)*(1-p1w);

  for(i=1; i<=NEMit; i++)         /* for NEMititerations of the EM */
  {
    /* printf("%04f, %0.4f, %0.4f, %0.4f \n", p11, p12, p21, p22); */
    tmp = (n1212*p11*p22)/(p11*p22 + p12*p21);
    p11 = (n1111 +.5*(n1112 + n1211 + tmp))/ss;

    /* Now, enforce additional functional contraints btwn D and    */
    /* Dmax. I think this step was an oversight in the original    */
    /* equations by Ken Lange, and it can lead to unrealistic D'   */
    /* and rsq estimates for a fintie sequence of EM iterations,   */
    /* irrespective of the presence or absence of local maxima.    */

    D = (p11 - p1w*pw1);
    if(  D > Dmaxpos ) p11 =  Dmaxpos + p1w * pw1;
    if( -D > Dmaxneg ) p11 = -Dmaxneg + p1w * pw1;

    p12 = p1w - p11;
    p21 = pw1 - p11;
    p22 = 1 - p11 - p12 - p21;
  }
  D = (p11 - p1w*pw1);

  /* Note: |D'| is constant for all haps: 1-1, 1-2, 2-1, and 2-2, so if this is all     */
  /*       you want (i.e. a summary of the correlation), you can ignore the coupling.   */
  /*       However, if you want to simulate a realistic multilocus linkage data set     */ 
  /*       using Haplodrop, then to ensure that allele freqs correspond to the right    */ 
  /*       haplotype (i.e. the haplotype with D'>0), then the coupling must be taken    */
  /*       into account.                                                                */

  if( cstat == 1 && D >= 0 )      /* 1 at 1 was coupled b4 and is now coupled w/ 1 at 2 */
  {
    frqs[0] = p1w;                /* report the '1' allele frqs at 1 and 2              */
    frqs[1] = pw1;
    tcstat = 1;
    Dmax = Dmaxpos;
  }
  if( cstat == 1 && D < 0 )       /* 1 at 1 was coupled b4 and is coupled w/ 2 at 2     */
  {
    frqs[0] = p1w;                /* report the '1' allele frq at 1                     */
    frqs[1] = 1 - pw1;            /* report the '2' allele frq at 2                     */
    tcstat = 2;
    Dmax = Dmaxneg;
  }
  if( cstat == 2 && D < 0 )       /* 2 at 1 was coupled b4 and is coupled w/ 1 at 2     */
  {
    frqs[0] = 1 - p1w;            /* report the '2' allele frq at 1                     */
    frqs[1] = pw1;                /* report the '1' allele frq at 2                     */
    tcstat = 1;
    Dmax = Dmaxneg;
  }
  if( cstat == 2 && D >= 0 )      /* 2 at 1 was coupled b4 and is coupled w/ 2 at 2     */
  {
    frqs[0] = 1 - p1w;            /* report the '2' allele frq at 1                     */
    frqs[1] = 1 - pw1;            /* report the '2' allele frq at 2                     */
    tcstat = 2;
    Dmax = Dmaxpos;
  }
  if( posld == 0 ) cstat = 1;     /* don't worry about coupling                         */
  else cstat = tcstat;            /* take coupling into account                         */

  if( !strcmp(flag,"rsq") ) ld = (D * D)/(p1w*(1-p1w)*pw1*(1-pw1));
  else ld = sqrt((D/Dmax)*(D/Dmax));

  /* printf("P(A) = %0.4f, P(B) = %0.4f \n", p1w, pw1);       */
  /* printf(" [%d - %d], %0.4f %0.4f \n", snp1, snp2, ld, D); */

  return(ld);
}

float comphet( int locus, int r )
{
  int pos1, pos2, i;
  float a1=0, a2=0, f1, f2, het;

  pos1 = 2 * locus + 5;
  pos2 = pos1 + 1;

  for( i=1; i<=r; i++)
  {
    if(PED[i][pos1]==1) a1++;
    if(PED[i][pos1]==2) a2++;
    if(PED[i][pos2]==1) a1++;
    if(PED[i][pos2]==2) a2++;
  }

  if( a1>0 || a2>0)                  /* if this snp is OBS'd in the sample     */
  {
    f1 = a1/(a1+a2);
    f2 = 1-f1;
    het = 1 - f1*f1 - f2*f2;
  }

  if( a1==0 && a2==0 ) het = 999;    /* if this snp is UNOBS'd in the sample   */

  return( het );
}

