#!/usr/bin/perl

## This code allows users to compute the ALOD curves, when the subsampling was 
## done across multiple cores. An alternative approach to computing the ALOD 
## would be to use the --custom option in eaglet.pl, although this option 
## seems a bit 'clunky' by comparison. 
##
## syntax: ./alod.pl --uniqid [prefix.xx]
## syntax: ./alod.pl --uniqid   abc.xx

## alod.pl-v1.0
## May 20, 2013

# use warnings;
# use strict; 

## ===================================================================
## SETUP  include headers
##        declare global vars
##        check command-line args
##        if successful, print general options and analysis parameters
## ===================================================================

require 'Utils/alod.ph';                            ## All Perl subroutines used by alod.sub() 

use Getopt::Long;

$pedfile = $datfile = $mapfile = $uniqid = '';                ## REQUIRED (for test, estimate, subsample)
   $tola = $modfile = '';                           ## OPTIONAL (both neeeded for parametric analysis)
   $rldt = $dldt = '';                              ## OPTIONAL     (Pick one)
   $test = $estimate = $subsample = '';             ## OPTIONAL     (Any subset)
   $meta = $ldinfo = '';                            ## OPTIONAL     (Any subset)
 $custom = '';                                      ## CANNOT BE USED WITH META 

$version  = "ALOD-v1.1"; 
$wtola    = 3;                                      ## DEFAULT : Use HLODs for parametric analysis
$het      = 0.01;                                   ## DEFAULT : LB for 1st snp heterozygosity 
$nboots   = 100;                                    ## DEFAULT : number of bootstrap samples
$nskip    = 0;                                      ## DEFAULT : number of adj snps to skip 
$nsubs    = 50;                                     ## DEFAULT : number of subsamples
$posld    = 0;                                      ## DEFAULT : ignore coupling 
$yn       = "No";                                   ## DEFAULT : yes/no variable
$seed     = int(rand(9000000)) + 1000001;           ## draw a random seed between for C-functions 
$bpedfile = "boot.ped";

$args = GetOptions ("p=s"         => \$pedfile,   ## string
                    "d=s"         => \$datfile,   ## string
		    "m=s"         => \$mapfile,   ## string
                    "model=s"     => \$modfile,   ## string
                    "para=s"      => \$tola,      ## string 
                    "ldmeasure=s" => \$ldmeasure, ## string
                    "uniqid=s"    => \$uniqid,    ## string
                    "seed=i"      => \$seed,      ## integer
                    "nsubs=i"     => \$nsubs,     ## integer
                    "nboots=i"    => \$nboots,    ## integer
                    "nskip=i"     => \$nskip,     ## integer
                    "posld"       => \$posld,     ## boolean
                    "subsample"   => \$subsample, ## boolean
                    "test"        => \$test,      ## boolean
                    "estimate"    => \$estimate,  ## boolean
                    "ldinfo"      => \$ldinfo,    ## boolean
                    "meta"        => \$meta,      ## boolean
                    "custom"      => \$custom,    ## boolean
                    "dldt=f"      => \$dldt,      ## decimal
                    "rldt=f"      => \$rldt);     ## decimal

$uniq = substr($uniqid,0,-3);
if( $tola eq "lod" ){ $wtola = 1; }
if( $posld ){ $yn = "Yes"; }
system("clear");

## ==================
## BEGIN main program
## ==================

$nsubs = 0;  ## alod.pl should determine the number of subsamples since different 
             ## data sets may have used different numbers of subsamples.
my $string = my $MAE = "";
my @filearray = ();
my $ulodsdir = "Results/Test/Unaligned-Lods-$uniq";
opendir(DIR, $ulodsdir) or die "Could not open: $ulodsdir";
while (my $filenames = readdir(DIR)) 
{
  # Use a regular expression to ignore files beginning with a period
  next if ($filenames =~ m/^\./);
  $string = $ulodsdir."/$filenames";
  push(@filearray, $string);
  $nsubs++;
}
closedir(DIR);
push(@filearray, 0);  ## -1 -> ESTIMATE; 0 -> ALOD; 1 -> META-LOD; 2 -> CUSTOM 
print " Computing the ALOD curve based on $nsubs subsamples...\n";
push( @filearray, $uniqid ); 
print @filearray;
$MAE = alod( @filearray );
print "\n\n";
print " ========   ====\n";
print " Position   ALOD\n";
print " ========   ====\n\n";
system("paste -d ' ' Results/Test/cgrid-$uniqid Results/Test/alod-$uniqid");
print "\n";
system("cat Results/Test/mae-$uniqid");
print "\n\n";
print "\n THANK YOU for using $version \n\n";
system("paste -d ' ' Results/Test/cgrid-$uniqid Results/Test/alod-$uniqid > Lods/$uniq-lod");

## ================
## END main program
## ================
